#include <sys/param.h>
#include <sys/kernel.h>
#include <sys/module.h>
#include <sys/sysctl.h>
#include <sys/systm.h>
#include <sys/conf.h>
#include <sys/ioccom.h>

#ifndef __amd64__
#error "only works on amd64"
#endif

extern void test_intr_blocking1(u_long iterations);
extern void test_intr_blocking2(u_long iterations);

#define	INTR_BLOCKING_DOIT1	_IOW('x', 57, u_long)
#define	INTR_BLOCKING_DOIT2	_IOW('x', 58, u_long)

static int
intr_blocking_ioctl(struct cdev *cdev, u_long cmd, caddr_t data, int fflag,
    struct thread *td)
{
	u_long iterations;
	int error;

	error = 0;
	if (cmd == INTR_BLOCKING_DOIT1) {
		iterations = *(u_long *)data;
		test_intr_blocking1(iterations);
	} else if (cmd == INTR_BLOCKING_DOIT2) {
		iterations = *(u_long *)data;
		test_intr_blocking2(iterations);
	} else {
		error = ENOTTY;
	}

	return (error);
}

static struct cdevsw intr_blocking_sw = {
	.d_name = "intr_blocking",
	.d_version = D_VERSION,
	.d_ioctl = intr_blocking_ioctl
};

static struct cdev *intr_blocking_cdev;

static int
intr_blocking_modevent(module_t mod, int type, void *unused)
{
	int error;

        switch (type) {
        case MOD_LOAD:
		error = make_dev_p(MAKEDEV_CHECKNAME, &intr_blocking_cdev,
		    &intr_blocking_sw, NULL, UID_ROOT, GID_WHEEL, 0600,
		    "intr_blocking");
		KASSERT(error == 0, ("%s: make_dev error %d", __func__, error));
		break;
	case MOD_UNLOAD:
		KASSERT(intr_blocking_cdev, ("%s: cdev is NULL", __func__));
		destroy_dev_sched_cb(intr_blocking_cdev, NULL, NULL);
		intr_blocking_cdev = NULL;
		break;
	default:
		return (EINVAL);
	}

	return (0);
}

static moduledata_t intr_blocking_mod_data= {
	"intr_blocking",
	intr_blocking_modevent,
};

DECLARE_MODULE(intr_blocking, intr_blocking_mod_data, SI_SUB_SMP + 1,
    SI_ORDER_ANY);
