#include <machine/asmacros.h>

#define	VENTER	push %rbp; mov %rsp, %rbp
#define	VLEAVE	pop %rbp

/*
 * void test_intr_blocking1(u_long iterations)
 * %rdi: iterations
 */
ENTRY(test_intr_blocking1)
	VENTER
	pushfq			/* save %rflags */

	/*
	 * Clear RFLAGS.IF so that the next "sti" is guaranteed to cause
	 * interrupt blocking for one instruction after it executes.
	 */
	cli
1:
	cmpq $0, %rdi
	je 2f
	subq $1, %rdi

	sti
	outl %eax, $0x80
	/*
	 * The interrupt blocking due to STI should be cleared after the
	 * "outl" emulation.
	 *
	 * In the loop between the labels "1" and "2" this is the only
	 * instruction boundary where interrupts can be injected.
	 */
	cli
	jmp 1b
2:
	popfq			/* restore %rflags */
	VLEAVE
	ret
END(test_intr_blocking1)

/*
 * void test_intr_blocking2(u_long iterations)
 * %rdi: iterations
 */
ENTRY(test_intr_blocking2)
	VENTER
	pushfq			/* save %rflags */

	/*
	 * Clear RFLAGS.IF so that the next "sti" is guaranteed to cause
	 * interrupt blocking for one instruction after it executes.
	 */
	cli
1:
	cmpq $0, %rdi
	je 2f
	subq $1, %rdi

	sti
	xor %eax, %eax
	/*
	 * The interrupt blocking due to STI should be cleared after the
	 * "xor" instruction. If the hypervisor has enabled interrupt
	 * window exiting the processor will do a VM-exit at this point.
	 *
	 * In the loop between the labels "1" and "2" this is the only
	 * instruction boundary where interrupts can be injected.
	 */
	cli
	jmp 1b
2:
	popfq			/* restore %rflags */
	VLEAVE
	ret
END(test_intr_blocking2)
